#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

from toscaparser.common.exception import ExceptionCollector
from toscaparser.common.exception import URLException
from toscaparser.common.exception import ValidationError
from toscaparser.tosca_template import ToscaTemplate
from toscaparser.utils.gettextutils import _
from translator.common.exception import UnsupportedTypeError
from translator.common.utils import TranslationUtils
from translator.hot.tosca_translator import TOSCATranslator
from translator.tests.base import TestCase


class ToscaHotTranslationTest(TestCase):

    def _test_successful_translation(self, tosca_file, hot_files, params=None):
        if not params:
            params = {}
        if not isinstance(hot_files, list):
            hot_files = [hot_files]
        diff = TranslationUtils.compare_tosca_translation_with_hot(tosca_file,
                                                                   hot_files,
                                                                   params)
        self.assertEqual({}, diff, '<difference> : ' +
                         json.dumps(diff, indent=4, separators=(', ', ': ')))

    def _test_failed_translation(self, tosca_file, hot_file, params, msg,
                                 msg_path, error_raise, error_collect):
        if msg_path:
            path = os.path.normpath(os.path.join(
                os.path.dirname(os.path.realpath(__file__)), tosca_file))
            msg = msg % path
        self.assertRaises(
            error_raise,
            TranslationUtils.compare_tosca_translation_with_hot,
            tosca_file, hot_file, params)
        ExceptionCollector.assertExceptionMessage(error_collect, msg)

    def test_hot_translate_single_server(self):
        tosca_file = '../tests/data/tosca_single_server.yaml'
        hot_file = '../tests/data/hot_output/hot_single_server.yaml'
        params = {'cpus': 1}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_single_server_with_defaults(self):
        tosca_file = \
            '../tests/data/tosca_single_server_with_defaults.yaml'

        hot_file_with_input = '../tests/data/hot_output/' \
            'hot_single_server_with_defaults_with_input.yaml'
        params1 = {'cpus': '1'}
        self._test_successful_translation(tosca_file, hot_file_with_input,
                                          params1)

        hot_file_without_input = '../tests/data/hot_output/' \
            'hot_single_server_with_defaults_without_input.yaml'
        params2 = {}
        self._test_successful_translation(tosca_file, hot_file_without_input,
                                          params2)

    def test_hot_translate_wordpress_single_instance(self):
        tosca_file = '../tests/data/tosca_single_instance_wordpress.yaml'
        hot_file = '../tests/data/hot_output/' \
            'hot_single_instance_wordpress.yaml'
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_helloworld(self):
        tosca_file = '../tests/data/tosca_helloworld.yaml'
        hot_file = '../tests/data/hot_output/hot_hello_world.yaml'
        self._test_successful_translation(tosca_file, hot_file)

    def test_hot_translate_host_assignment(self):
        tosca_file = '../tests/data/test_host_assignment.yaml'
        hot_file = '../tests/data/hot_output/hot_host_assignment.yaml'
        self._test_successful_translation(tosca_file, hot_file)

    def test_hot_translate_elk(self):
        tosca_file = '../tests/data/tosca_elk.yaml'
        hot_file = '../tests/data/hot_output/hot_elk.yaml'
        params = {'github_url':
                  'http://github.com/paypal/rest-api-sample-app-nodejs.git',
                  'my_cpus': 4}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_nodejs_mongodb_two_instances(self):
        tosca_file = '../tests/data/tosca_nodejs_mongodb_two_instances.yaml'
        hot_file = '../tests/data/hot_output/' \
                   'hot_nodejs_mongodb_two_instances.yaml'
        params = {'github_url':
                  'http://github.com/paypal/rest-api-sample-app-nodejs.git',
                  'my_cpus': 4}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_blockstorage_with_attachment(self):
        tosca_file = '../tests/data/storage/' \
                     'tosca_blockstorage_with_attachment.yaml'
        hot_file = '../tests/data/hot_output/storage/' \
                   'hot_blockstorage_with_attachment.yaml'
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '2000 MB',
                  'storage_snapshot_id': 'ssid'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_blockstorage_with_custom_relationship_type(self):
        tosca_file = '../tests/data/storage/' \
                     'tosca_blockstorage_with_custom_relationship_type.yaml'
        hot_file = '../tests/data/hot_output/storage/' \
                   'hot_blockstorage_with_custom_relationship_type.yaml'
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_blockstorage_with_relationship_template(self):
        tosca_file = '../tests/data/storage/' \
                     'tosca_blockstorage_with_relationship_template.yaml'
        hot_file = '../tests/data/hot_output/storage/' \
                   'hot_blockstorage_with_relationship_template.yaml'
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '1 GB'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_blockstorage_with_attachment_notation1(self):
        tosca_file = '../tests/data/storage/' \
                     'tosca_blockstorage_with_attachment_notation1.yaml'
        hot_file1 = '../tests/data/hot_output/storage/' \
                    'hot_blockstorage_with_attachment_notation1_alt1.yaml'
        hot_file2 = '../tests/data/hot_output/storage/' \
                    'hot_blockstorage_with_attachment_notation1_alt2.yaml'
        params = {'cpus': 1,
                  'storage_location': 'some_folder',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}

        try:
            self._test_successful_translation(tosca_file, hot_file1, params)
        except Exception:
            self._test_successful_translation(tosca_file, hot_file2, params)

    def test_hot_translate_blockstorage_with_attachment_notation2(self):
        tosca_file = '../tests/data/storage/' \
                     'tosca_blockstorage_with_attachment_notation2.yaml'
        hot_file1 = '../tests/data/hot_output/storage/' \
                    'hot_blockstorage_with_attachment_notation2_alt1.yaml'
        hot_file2 = '../tests/data/hot_output/storage/' \
                    'hot_blockstorage_with_attachment_notation2_alt2.yaml'
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}
        try:
            self._test_successful_translation(tosca_file, hot_file1, params)
        except Exception:
            self._test_successful_translation(tosca_file, hot_file2, params)

    def test_hot_translate_multiple_blockstorage_with_attachment(self):
        tosca_file = '../tests/data/storage/' \
                     'tosca_multiple_blockstorage_with_attachment.yaml'
        hot_file1 = '../tests/data/hot_output/storage/' \
                    'hot_multiple_blockstorage_with_attachment_alt1.yaml'
        hot_file2 = '../tests/data/hot_output/storage/' \
                    'hot_multiple_blockstorage_with_attachment_alt2.yaml'
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}
        try:
            self._test_successful_translation(tosca_file, hot_file1, params)
        except Exception:
            self._test_successful_translation(tosca_file, hot_file2, params)

    def test_hot_translate_multiple_blockstorage_w_multiple_attachment(self):
        tosca_file = '../tests/data/storage/' \
                     'tosca_multiple_blockstorage_w_multiple_attachment.yaml'
        hot_file = '../tests/data/hot_output/storage/' \
                   'hot_multiple_blockstorage_w_multiple_attachment.yaml'
        params = {'cpus': 1,
                  'storage_location1': '/dev/vdb',
                  'storage_location2': '/dev/vdc',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_single_object_store(self):
        tosca_file = '../tests/data/storage/tosca_single_object_store.yaml'
        hot_file = '../tests/data/hot_output/hot_single_object_store.yaml'
        params = {'objectstore_name': 'myobjstore'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_one_server_one_network(self):
        tosca_file = '../tests/data/network/tosca_one_server_one_network.yaml'
        hot_file = '../tests/data/hot_output/network/' \
                   'hot_one_server_one_network.yaml'
        params = {'network_name': 'private_net'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_server_on_existing_network(self):
        tosca_file = '../tests/data/network/' \
                     'tosca_server_on_existing_network.yaml'
        hot_file = '../tests/data/hot_output/network/' \
                   'hot_server_on_existing_network.yaml'
        params = {'network_name': 'private_net'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_two_servers_one_network(self):
        tosca_file = '../tests/data/network/tosca_two_servers_one_network.yaml'
        hot_file = '../tests/data/hot_output/network/' \
                   'hot_two_servers_one_network.yaml'
        params = {'network_name': 'my_private_net',
                  'network_cidr': '10.0.0.0/24',
                  'network_start_ip': '10.0.0.100',
                  'network_end_ip': '10.0.0.150'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_one_server_three_networks(self):
        tosca_file = '../tests/data/network/' \
                     'tosca_one_server_three_networks.yaml'
        hot_file = '../tests/data/hot_output/network/' \
                   'hot_one_server_three_networks.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_software_component(self):
        tosca_file = '../tests/data/tosca_software_component.yaml'
        hot_file = '../tests/data/hot_output/hot_software_component.yaml'
        params = {'cpus': '1',
                  'download_url': 'http://www.software.com/download'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_software_component_multiple_hosts(self):
        tosca_file = '../tests/data/tosca_software_component'\
            '_multiple_hosts.yaml'
        hot_file = '../tests/data/hot_output/hot_software_component'\
            '_multiple_hosts.yaml'
        params = {'cpus': '1',
                  'download_url': 'http://www.software.com/download'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_web_application(self):
        tosca_file = '../tests/data/tosca_web_application.yaml'
        hot_file = '../tests/data/hot_output/hot_web_application.yaml'
        params = {'cpus': '2', 'context_root': 'my_web_app'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_template_with_url_import(self):
        tosca_file = '../tests/data/' \
                     'tosca_single_instance_wordpress_with_url_import.yaml'
        hot_file = '../tests/data/hot_output/' \
                   'hot_single_instance_wordpress.yaml'
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_template_by_url_with_local_import(self):
        tosca_file = 'https://raw.githubusercontent.com/openstack/' \
                     'heat-translator/master/translator/tests/data/' \
                     'tosca_single_instance_wordpress.yaml'
        hot_file = '../tests/data/hot_output/' \
                   'hot_single_instance_wordpress.yaml'
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_template_by_url_with_local_abspath_import(self):
        tosca_file = 'https://raw.githubusercontent.com/openstack/' \
                     'heat-translator/master/translator/tests/data/' \
                     'tosca_single_instance_wordpress_with_local_abspath' \
                     '_import.yaml'
        hot_file = '../tests/data/hot_output/' \
                   'hot_single_instance_wordpress.yaml'
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        expected_msg = _('Absolute file name "/tmp/wordpress.yaml" cannot be '
                         'used in a URL-based input template "https://raw.'
                         'githubusercontent.com/openstack/heat-translator/'
                         'master/translator/tests/data/tosca_single_instance_'
                         'wordpress_with_local_abspath_import.yaml".')
        msg_path = False
        self._test_failed_translation(tosca_file, hot_file, params,
                                      expected_msg, msg_path, ValidationError,
                                      ImportError)

    def test_hot_translate_template_by_url_with_url_import(self):
        tosca_url = 'https://raw.githubusercontent.com/openstack/' \
                    'heat-translator/master/translator/tests/data/' \
                    'tosca_single_instance_wordpress_with_url_import.yaml'
        hot_file = '../tests/data/hot_output/' \
                   'hot_single_instance_wordpress.yaml'
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._test_successful_translation(tosca_url, hot_file, params)

    def test_translate_hello_world_csar(self):
        tosca_file = '../tests/data/csar_hello_world.zip'
        hot_file = '../tests/data/hot_output/hot_hello_world.yaml'
        self._test_successful_translation(tosca_file, hot_file)

    def test_translate_single_instance_wordpress_csar(self):
        tosca_file = '../tests/data/csar_single_instance_wordpress.zip'
        hot_file = '../tests/data/hot_output/' \
                   'hot_single_instance_wordpress_from_csar.yaml'
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_translate_elk_csar_from_url(self):
        tosca_file = 'https://github.com/openstack/heat-translator/raw/' \
                     'master/translator/tests/data/csar_elk.zip'
        hot_file = '../tests/data/hot_output/hot_elk_from_csar.yaml'
        params = {'github_url':
                  'http://github.com/paypal/rest-api-sample-app-nodejs.git',
                  'my_cpus': 4}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_translate_csar_not_zip(self):
        tosca_file = '../tests/data/csar_not_zip.zip'
        hot_file = ''
        params = {}
        expected_msg = _('"%s" is not a valid zip file.')
        msg_path = True
        self._test_failed_translation(tosca_file, hot_file, params,
                                      expected_msg, msg_path, ValidationError,
                                      ValidationError)

    def test_translate_csar_metadata_not_yaml(self):
        tosca_file = '../tests/data/csar_metadata_not_yaml.zip'
        hot_file = ''
        params = {}
        expected_msg = _('The file "TOSCA-Metadata/TOSCA.meta" in the CSAR '
                         '"%s" does not contain valid YAML content.')
        msg_path = True
        self._test_failed_translation(tosca_file, hot_file, params,
                                      expected_msg, msg_path, ValidationError,
                                      ValidationError)

    def test_translate_csar_wrong_metadata_file(self):
        tosca_file = '../tests/data/csar_wrong_metadata_file.zip'
        hot_file = ''
        params = {}
        expected_msg = _('"%s" is not a valid CSAR as it does not contain the '
                         'required file "TOSCA.meta" in the folder '
                         '"TOSCA-Metadata".')
        msg_path = True
        self._test_failed_translation(tosca_file, hot_file, params,
                                      expected_msg, msg_path, ValidationError,
                                      ValidationError)

    def test_translate_csar_wordpress_invalid_import_path(self):
        tosca_file = '../tests/data/csar_wordpress_invalid_import_path.zip'
        hot_file = ''
        params = {}
        expected_msg = _('Import '
                         '"Invalid_import_path/wordpress.yaml" is not valid.')
        msg_path = False
        self._test_failed_translation(tosca_file, hot_file, params,
                                      expected_msg, msg_path, ValidationError,
                                      ImportError)

    def test_translate_csar_wordpress_invalid_script_url(self):
        tosca_file = '../tests/data/csar_wordpress_invalid_script_url.zip'
        hot_file = ''
        params = {}
        expected_msg = _('The resource at '
                         '"https://raw.githubusercontent.com/openstack/'
                         'heat-translator/master/translator/tests/data/'
                         'custom_types/wordpress1.yaml" cannot be accessed.')
        msg_path = False
        self._test_failed_translation(tosca_file, hot_file, params,
                                      expected_msg, msg_path, ValidationError,
                                      URLException)

    def test_hot_translate_flavor_image(self):
        tosca_file = '../tests/data/test_tosca_flavor_and_image.yaml'
        hot_file = '../tests/data/hot_output/hot_flavor_and_image.yaml'
        self._test_successful_translation(tosca_file, hot_file)

    def test_hot_translate_flavor_image_params(self):
        tosca_file = '../tests/data/test_tosca_flavor_and_image.yaml'
        hot_file = '../tests/data/hot_output/hot_flavor_and_image_params.yaml'
        params = {'key_name': 'paramkey'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_custom_type(self):
        tosca_file = '../tests/data/test_tosca_custom_type.yaml'
        hot_file = '../tests/data/hot_output/' \
            'hot_custom_type.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_custom_type_with_override(self):
        tosca_file = '../tests/data/test_tosca_custom_type_with_override.yaml'
        hot_file = '../tests/data/hot_output/' \
            'hot_custom_type_with_override.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_custom_type_with_param_override(self):
        tosca_file = '../tests/data/test_tosca_custom_type_with_override.yaml'
        hot_file = '../tests/data/hot_output/' \
            'hot_custom_type_with_param_override.yaml'
        params = {'install_path': '/home/custom/from/cli'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_artifact(self):
        tosca_file = '../tests/data/test_tosca_artifact.yaml'
        hot_file = '../tests/data/hot_output/' \
            'hot_artifact.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_without_tosca_os_version(self):
        tosca_file = '../tests/data/' \
            'test_single_server_without_optional_version_prop.yaml'
        hot_file = '../tests/data/hot_output/' \
            'hot_single_server_without_tosca_os_version.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_helloworld_with_userkey(self):
        tosca_file = '../tests/data/tosca_helloworld.yaml'
        hot_file = '../tests/data/hot_output/hot_hello_world_userkey.yaml'
        params = {'key_name': 'userkey'}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_custom_networks_nodes_inline(self):
        tosca_file = '../tests/data/network/' \
                     'test_tosca_custom_network_nodes_inline.yaml'
        hot_file = '../tests/data/hot_output/network/' \
                   'hot_custom_network_nodes.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_custom_networks_nodes_imports(self):
        tosca_file = '../tests/data/network/' \
                     'test_tosca_custom_network_nodes_imports.yaml'
        hot_file = '../tests/data/hot_output/network/' \
                   'hot_custom_network_nodes.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_nfv_sample(self):
        tosca_file = '../tests/data/nfv/test_tosca_nfv_sample.yaml'
        hot_file = '../tests/data/hot_output/nfv/hot_nfv_sample.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_placement_policy_default_affinity(self):
        tosca_file = '../tests/data/policies/tosca_policies.yaml'
        hot_file = '../tests/data/hot_output/policies/hot_policies.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_placement_policy_affinity(self):
        tosca_file = ('../tests/data/nfv/'
                      'tosca_placement_policy_affinity.yaml')
        hot_file = '../tests/data/hot_output/policies/hot_policies.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_placement_policy_anti_affinity(self):
        tosca_file = ('../tests/data/nfv/'
                      'tosca_placement_policy_anti_affinity.yaml')
        hot_file = ('../tests/data/hot_output/nfv/'
                    'hot_policy_anti_affinity.yaml')
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_placement_policy_soft_affinity(self):
        tosca_file = ('../tests/data/nfv/'
                      'tosca_placement_policy_soft_affinity.yaml')
        hot_file = ('../tests/data/hot_output/nfv/'
                    'hot_policy_soft_affinity.yaml')
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_script_types(self):
        tosca_file = '../tests/data/interfaces/test_tosca_script_types.yaml'
        hot_file = '../tests/data/hot_output/hot_script_types.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_interface_on_compute(self):
        tosca_file = '../tests/data/interfaces/' \
                     'test_tosca_interface_on_compute.yaml'
        hot_file = '../tests/data/hot_output/interfaces/' \
                   'hot_interface_on_compute.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_get_functions_semantic(self):
        tosca_file = '../tests/data/test_tosca_get_functions_semantic.yaml'
        hot_file = '../tests/data/hot_output/hot_get_functions_semantic.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_exchange_public_ssh_key(self):
        tosca_file = '../tests/data/tosca_exchange_public_ssh_key.yaml'
        hot_file = '../tests/data/hot_output/hot_exchange_public_ssh_key.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_scaling_policy(self):
        tosca_file = '../tests/data/autoscaling/tosca_autoscaling.yaml'
        hot_files = [
            '../tests/data/hot_output/autoscaling/hot_autoscaling.yaml',
            '../tests/data/hot_output/autoscaling/asg_res.yaml',
            ]
        params = {}
        self._test_successful_translation(tosca_file, hot_files, params)

    def test_translate_unsupported_tosca_type(self):
        tosca_file = '../tests/data/test_tosca_unsupported_type.yaml'
        tosca_tpl = os.path.normpath(os.path.join(
            os.path.dirname(os.path.abspath(__file__)), tosca_file))
        params = {}
        expected_msg = _('Type "tosca.nodes.LoadBalancer" is valid TOSCA '
                         'type but translation support is not yet available.')
        tosca = ToscaTemplate(tosca_tpl, params, True)
        err = self.assertRaises(UnsupportedTypeError,
                                TOSCATranslator(tosca, params)
                                .translate)
        self.assertEqual(expected_msg, err.__str__())

    def test_translate_unsupported_tosca_policy_type(self):
        tosca_file = ('../tests/data/nfv/'
                      'test_tosca_unsupported_policy_type.yaml')
        tosca_tpl = os.path.normpath(os.path.join(
            os.path.dirname(os.path.abspath(__file__)), tosca_file))
        params = {}
        expected_msg = _('Type "tosca.policies.tacker.ABC" is valid TOSCA '
                         'type but translation support is not yet available.')
        tosca = ToscaTemplate(tosca_tpl, params, True)
        err = self.assertRaises(UnsupportedTypeError,
                                TOSCATranslator(tosca, params)
                                .translate)
        self.assertEqual(expected_msg, err.__str__())

    def test_hot_translate_cluster_scaling_policy(self):
        tosca_file = '../tests/data/autoscaling/tosca_cluster_autoscaling.yaml'
        hot_file = '../tests/data/hot_output/autoscaling/' \
                   'hot_cluster_autoscaling.yaml'
        params = {}
        self._test_successful_translation(tosca_file, hot_file, params)

    def test_hot_translate_nfv_scaling(self):
        tosca_file = '../tests/data/nfv/test_tosca_nfv_autoscaling.yaml'
        hot_files = [
            '../tests/data/hot_output/nfv/hot_tosca_nfv_autoscaling.yaml',
            '../tests/data/hot_output/nfv/SP1_res.yaml',
            ]
        params = {}
        self._test_successful_translation(tosca_file, hot_files, params)

    def test_hot_translate_nfv_scaling_with_params(self):
        tosca_file = '../tests/data/nfv/test_tosca_nfv_autoscaling_with_' \
                     'params.yaml'
        hot_files = [
            '../tests/data/hot_output/nfv/hot_tosca_nfv_autoscaling_with_'
            'param.yaml',
            '../tests/data/hot_output/nfv/SP_res.yaml',
        ]
        params = {'image_name': 'cirros-0.3.5-x86_64-disk',
                  'flavor': 'm1.tiny'}
        self._test_successful_translation(tosca_file, hot_files, params)

    def test_hot_translate_mon_scaling_policy(self):
        tosca_file = '../tests/data/monitoring/tosca_monitoring_scaling.yaml'
        hot_files = [
            '../tests/data/hot_output/monitoring/hot_monitoring_scaling.yaml',
            '../tests/data/hot_output/monitoring/asg_res.yaml',
        ]
        params = {}
        self._test_successful_translation(tosca_file, hot_files, params)

    def test_hot_translate_reservation_policy(self):
        tosca_file = '../tests/data/reservation/tosca-vnfd-reservation-id.yaml'
        hot_files = [
            '../tests/data/hot_output/reservation/'
            'hot_reservation_scaling.yaml',
            '../tests/data/hot_output/reservation/SP_RSV_res.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from translator.hot.syntax.hot_resource import HotResource

# Name used to dynamically load appropriate map class.
TARGET_CLASS_NAME = 'ToscaFloatingIP'
TOSCA_LINKS_TO = 'tosca.relationships.network.LinksTo'


class ToscaFloatingIP(HotResource):
    '''Translate TOSCA node type tosca.nodes.network.FloatingIP'''

    toscatype = 'tosca.nodes.network.FloatingIP'

    def __init__(self, nodetemplate, csar_dir=None):
        super(ToscaFloatingIP, self).__init__(nodetemplate,
                                              type='OS::Neutron::FloatingIP',
                                              csar_dir=csar_dir)

    def handle_properties(self):
        tosca_props = self.get_tosca_props()
        fip_props = {}
        for key, value in tosca_props.items():
            fip_props[key] = value

        links_to = None
        for rel, node in self.nodetemplate.relationships.items():
            if not links_to and rel.is_derived_from(TOSCA_LINKS_TO):
                links_to = node
                for hot_resource in self.depends_on_nodes:
                    if links_to.name == hot_resource.name:
                        self.depends_on.remove(hot_resource)
                        break
                fip_props['port_id'] =\
                    '{ get_resource: %s }' % (links_to.name)

        self.properties = fip_props



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

'''
Exceptions for the TOSCA Translator package.
'''

from toscaparser.common.exception import TOSCAException
from toscaparser.utils.gettextutils import _


class ConfFileParseError(TOSCAException):
    msg_fmt = _('%(message)s')


class ConfOptionNotDefined(TOSCAException):
    msg_fmt = _('Option %(key)s in section %(section)s '
                'is not defined in conf file')


class ConfSectionNotDefined(TOSCAException):
    msg_fmt = _('Section %(section)s is not defined in conf file')


class ToscaModImportError(TOSCAException):
    msg_fmt = _('Unable to import module %(mod_name)s. '
                'Check to see that it exists and has no '
                'language definition errors.')


class ToscaClassImportError(TOSCAException):
    msg_fmt = _('Unable to import class %(name)s in '
                'module %(mod_name)s. Check to see that it '
                'exists and has no language definition errors.')


class UnsupportedTypeError(TOSCAException):
    msg_fmt = _('Type "%(type)s" is valid TOSCA type but translation '
                'support is not yet available.')


class ToscaClassAttributeError(TOSCAException):
    msg_fmt = _('Class attribute referenced not found. '
                '%(message)s. Check to see that it is defined.')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from translator.osc import utils

DEFAULT_TRANSLATOR_API_VERSION = '1'
API_VERSION_OPTION = 'os_translator_api_version'
API_NAME = 'translator'
API_VERSIONS = {
    '1': 'translator.v1.client.Client',
}


def make_client(instance):
    # NOTE(stevemar): We don't need a client because
    # heat-translator itself is a command line tool
    pass


def build_option_parser(parser):
    """Hook to add global options."""

    parser.add_argument(
        '--os-translator-api-version',
        metavar='<translator-api-version>',
        default=utils.env(
            'OS_TRANSLATOR_API_VERSION',
            default=DEFAULT_TRANSLATOR_API_VERSION),
        help='Translator API version, default=' +
             DEFAULT_TRANSLATOR_API_VERSION +
             ' (Env: OS_TRANSLATOR_API_VERSION)')
    return parser



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import sys
from unittest import mock


class FakeApp(object):
    def __init__(self):
        self.client_manager = None
        self.stdin = sys.stdin
        self.stdout = sys.stdout
        self.stderr = sys.stderr

        self.cloud = mock.Mock()
        self.cloud.get_session.return_value = None


class FakeClientManager(object):
    def __init__(self):
        self.compute = None
        self.identity = None
        self.image = None
        self.object_store = None
        self.volume = None
        self.network = None
        self.session = None



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import yaml

from toscaparser.nodetemplate import NodeTemplate
from toscaparser.policy import Policy
from toscaparser.tests.base import TestCase
import toscaparser.utils.yamlparser

from translator.hot.syntax.hot_parameter import HotParameter
from translator.hot.tosca.tosca_compute import ToscaCompute
from translator.hot.tosca.tosca_policies_scaling import ToscaAutoscaling

HOT_TEMPLATE_VERSION = '2013-05-23'


class AutoscalingTest(TestCase):

    def _tosca_scaling_test(self, tpl_snippet, expectedprops,
                            hot_template_parameters=None):
        nodetemplates = (toscaparser.utils.yamlparser.
                         simple_parse(tpl_snippet)['node_templates'])
        policies = (toscaparser.utils.yamlparser.
                    simple_parse(tpl_snippet)['policies'])
        name = list(nodetemplates.keys())[0]
        policy_name = list(policies[0].keys())[0]
        for policy in policies:
            tpl = policy[policy_name]
            targets = tpl["targets"]
            properties = tpl["properties"]
        try:
            nodetemplate = NodeTemplate(name, nodetemplates)
            toscacompute = ToscaCompute(nodetemplate)
            toscacompute.handle_properties()
            policy = Policy(policy_name, tpl, targets,
                            properties, "node_templates")
            toscascaling = ToscaAutoscaling(
                policy, hot_template_parameters=hot_template_parameters)
            parameters = toscascaling.handle_properties([toscacompute])
            if hot_template_parameters:
                substack_template = toscascaling.extract_substack_templates(
                    "output.yaml", HOT_TEMPLATE_VERSION)
                actual_nested_resource = yaml.load(
                    substack_template['SP1_res.yaml'])
                self.assertEqual(expectedprops,
                                 actual_nested_resource)
            else:
                self.assertEqual(parameters[0].properties, expectedprops)
        except Exception:
            raise

    def test_compute_with_scaling(self):
        tpl_snippet = '''
        node_templates:
          my_server_1:
            type: tosca.nodes.Compute
            capabilities:
              host:
               properties:
                num_cpus: 2
                disk_size: 10 GB
                mem_size: 512 MB
              os:
               properties:
           # host Operating System image properties
                  architecture: x86_64
                  type: Linux
                  distribution: RHEL
                  version: 6.5
        policies:
          - asg:
              type: tosca.policies.Scaling
              description: Simple node autoscaling
              targets: [my_server_1]
              triggers:
                resize_compute:
                  description: trigger
                  condition:
                    constraint: utilization greater_than 50%
                    granularity: 60
                    evaluations: 1
                    aggregation_method: mean
                    resource_type: instance
                    comparison_operator: gt
              properties:
                min_instances: 2
                max_instances: 10
                default_instances: 3
                increment: 1
                cooldown: 60
              '''

        expectedprops = {'desired_capacity': 3,
                         'max_size': 10,
                         'min_size': 2,
                         'cooldown': 60,
                         'resource': {'type': 'asg_res.yaml'}}

        self._tosca_scaling_test(
            tpl_snippet,
            expectedprops)

    def test_scaling_nested_template_with_params(self):
        tpl_snippet = '''
        node_templates:
          VDU1:
            type: tosca.nodes.Compute
            properties:
              image: { get_input: image_name }
              flavor: { get_input: flavor }
              mgmt_driver: noop
              availability_zone: nova
              metadata: {metering.server_group: SG1}

        policies:
          - SP1:
              type: tosca.policies.Scaling
              targets: [VDU1]
              properties:
                increment: 1
                cooldown: 120
                min_instances: 1
                max_instances: 3
                default_instances: 1
            '''

        expected_nested_resource = {'heat_template_version':
                                    datetime.date(2013, 5, 23),
                                    'description':
                                        'Scaling template',
                                    'parameters':
                                        {'flavor':
                                         {'default': 'm1.tiny',
                                          'type': 'string',
                                          'description':
                                              'Flavor Information'
                                          },
                                         'image_name':
                                             {'default':
                                              'cirros-0.3.5-x86_64-disk',
                                              'type': 'string',
                                              'description': 'Image Name'
                                              }
                                         },
                                    'resources':
                                        {'VDU1':
                                             {'type':
                                              'OS::Nova::Server',
                                              'properties':
                                                  {'flavor': None,
                                                   'user_data_format':
                                                       'SOFTWARE_CONFIG'
                                                   }
                                              }
                                         }
                                    }

        flavor = HotParameter('flavor', 'string',
                              label=None,
                              description='Flavor Information',
                              default='m1.tiny',
                              hidden=None,
                              constraints=[])
        image = HotParameter('image_name', 'string',
                             label=None,
                             description='Image Name',
                             default='cirros-0.3.5-x86_64-disk',
                             hidden=None,
                             constraints=[])
        hot_template_parameters = [flavor, image]
        self._tosca_scaling_test(tpl_snippet,
                                 expected_nested_resource,
                                 hot_template_parameters)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from translator.common.utils import MemoryUnit
from translator.hot.syntax.hot_resource import HotResource

log = logging.getLogger('heat-translator')


# Name used to dynamically load appropriate map class.
TARGET_CLASS_NAME = 'ToscaNfvVnfVirtualLink'


class ToscaNfvVnfVirtualLink(HotResource):
    """Translate TOSCA node type tosca.nodes.nfv.VnfVirtualLink."""

    toscatype = 'tosca.nodes.nfv.VnfVirtualLink'
    SUBNET_SUFFIX = '_subnet'
    QOSPOLICY_SUFFIX = '_qospolicy'
    BANDWIDTH_SUFFIX = '_bandwidth'
    ip_map = {
        'ipv4': '4',
        'ipv6': '6',
    }

    def __init__(self, nodetemplate, csar_dir=None):

        # Check if it is an IP network
        tosca_props = {}
        for prop in nodetemplate.get_properties_objects():
            tosca_props[prop.name] = prop.value

        lp = tosca_props.get('connectivity_type').get('layer_protocols')
        self.ip_protocol = list(set(self.ip_map.keys()) & set(lp))

        # Branch by IP or not
        if self.ip_protocol:
            super(ToscaNfvVnfVirtualLink, self).__init__(
                nodetemplate,
                type='OS::Neutron::Net',
                csar_dir=csar_dir
            )
        else:
            super(ToscaNfvVnfVirtualLink, self).__init__(
                nodetemplate,
                csar_dir=csar_dir
            )
            log.warning(('Unsupported layer_protocols, '
                         'virtual_link_name:%s, protocol_name:%s')
                        % (self.name, lp))

    def handle_properties(self):

        # Branch by IP or not
        if self.ip_protocol:
            tosca_props = self.get_tosca_props()
            own_props = {}

            self.is_leaf = False
            for key, value in tosca_props.items():
                if key == 'vl_profile':
                    mbr = value['max_bitrate_requirements']
                    if 'leaf' in mbr:
                        max_bps = mbr['leaf']
                        # Convert to KiB
                        self.max_kbps = \
                            max_bps / MemoryUnit.UNIT_SIZE_DICT['KiB']
                        self.is_leaf = True
                    else:
                        log.warning('Can not set the required properties '
                                    'max_kbps on HOT.'
                                    'virtual_link_name:%s' % self.name)

            if self.is_leaf:
                own_props['qos_policy'] = '{ get_resource: %s%s }' % (
                    self.name,
                    self.QOSPOLICY_SUFFIX,
                )

            self.properties = own_props

        else:
            pass

    def handle_expansion(self):
        hot_resources = []

        # Branch by IP or not
        if self.ip_protocol:
            tosca_props = self.get_tosca_props()

            # subnet props
            subnet_props = {}
            subnet_props['ip_version'] = self.ip_map.get(self.ip_protocol[0])
            subnet_props['network'] = '{ get_resource: %s }' % (self.name)

            for key, value in tosca_props.items():
                if key == 'vl_profile':
                    if 'virtual_link_protocol_data' in value:
                        vlpd = value['virtual_link_protocol_data']
                        if 'l3_protocol_data' in vlpd[0]:
                            l3pd = vlpd[0]['l3_protocol_data']
                            subnet_props['cidr'] = l3pd['cidr']

            subnet_resource_name = self.name + self.SUBNET_SUFFIX

            hot_resources.append(
                HotResource(
                    self.nodetemplate,
                    type='OS::Neutron::Subnet',
                    name=subnet_resource_name,
                    properties=subnet_props,
                )
            )

            # qospolicy_props props
            qospolicy_props = {}

            qospolicy_resource_name = self.name + self.QOSPOLICY_SUFFIX

            # bandwidth props
            bandwidth_props = {}
            bandwidth_props['policy'] = '{ get_resource: %s%s }' % (
                self.name,
                self.QOSPOLICY_SUFFIX,
            )

            bandwidth_resource_name = self.name + self.BANDWIDTH_SUFFIX

            # Create QoSPolicy and QoSBandwidthLimitRule resources
            # only when max_bitrate_requirements has leaf property.
            if self.is_leaf:
                hot_resources.append(
                    HotResource(
                        self.nodetemplate,
                        type='OS::Neutron::QoSPolicy',
                        name=qospolicy_resource_name,
                        properties=qospolicy_props,
                    )
                )

                bandwidth_props['max_kbps'] = self.max_kbps
                hot_resources.append(
                    HotResource(
                        self.nodetemplate,
                        type='OS::Neutron::QoSBandwidthLimitRule',
                        name=bandwidth_resource_name,
                        properties=bandwidth_props,
                    )
                )

        else:
            pass

        return hot_resources



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import testtools
from unittest import mock

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import toscaparser.utils.yamlparser

from toscaparser.common.exception import ExceptionCollector
from toscaparser.common.exception import URLException
from toscaparser.common.exception import ValidationError
from toscaparser.utils.gettextutils import _
from translator.common.utils import CompareUtils
from translator.common.utils import YamlUtils
from translator.osc.v1.tests import fakes
from translator.osc.v1.tests import utils
from translator.osc.v1 import translate


class TestTranslateTemplate(testtools.TestCase):

    def setUp(self):
        super(TestTranslateTemplate, self).setUp()
        self.app = fakes.FakeApp()
        self.app.client_manager = fakes.FakeClientManager()
        self.app.client_manager.translator = None
        self.cmd = translate.TranslateTemplate(self.app, None)

    def check_parser(self, cmd, args, verify_args):
        cmd_parser = cmd.get_parser('check_parser')
        try:
            parsed_args = cmd_parser.parse_args(args)
        except SystemExit:
            raise Exception("Argument parse failed")
        for av in verify_args:
            attr, value = av
            if attr:
                self.assertIn(attr, parsed_args)
                self.assertEqual(getattr(parsed_args, attr), value)
        return parsed_args

    def _check_error(self, tosca_file, hot_file, params, assert_error,
                     expected_msg, c_error):
        arglist = ["--template-file", tosca_file,
                   "--template-type", "tosca"]
        parsed_args = self.check_parser(self.cmd, arglist, [])
        parsed_args.parameter = params
        self.assertRaises(assert_error, self.cmd.take_action,
                          parsed_args)
        ExceptionCollector.assertExceptionMessage(c_error, expected_msg)

    @mock.patch('sys.stdout', new_callable=StringIO)
    def _check_success(self, tosca_file, hot_file, params, mock_stdout):
        arglist = ["--template-file", tosca_file,
                   "--template-type", "tosca"]
        parsed_args = self.check_parser(self.cmd, arglist, [])
        parsed_args.parameter = params
        self.cmd.take_action(parsed_args)
        expected_output = YamlUtils.get_dict(hot_file)
        mock_stdout_yaml = "\n".join(mock_stdout.getvalue().split("\n"))
        actual_output = toscaparser.utils.yamlparser.simple_parse(
            mock_stdout_yaml)
        self.assertEqual({}, CompareUtils.diff_dicts(
            actual_output, expected_output))

    def test_osc_translate_single_server(self):
        tosca_file = utils.get_template_path("tosca_single_server.yaml")

        hot_file = utils.get_template_path("hot_output/hot_single_server.yaml")

        params = {'cpus': 1}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_single_server_defaults_with_input(self):
        tosca_file = utils.get_template_path(
            "tosca_single_server_with_defaults.yaml")

        hot_file = utils.get_template_path(
            "hot_output/hot_single_server_with_defaults_with_input.yaml")

        params = {'cpus': '1'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_single_server_defaults_without_input(self):
        tosca_file = utils.get_template_path(
            "tosca_single_server_with_defaults.yaml")

        hot_file = utils.get_template_path(
            "hot_output/hot_single_server_with_defaults_without_input.yaml")

        self._check_success(tosca_file, hot_file, {})

    def test_osc_translate_wordpress_single_instance(self):
        tosca_file = utils.get_template_path(
            "tosca_single_instance_wordpress.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_single_instance_wordpress.yaml")
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_helloworld(self):
        tosca_file = utils.get_template_path(
            "tosca_helloworld.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_hello_world.yaml")
        self._check_success(tosca_file, hot_file, {})

    def test_osc_translate_host_assignment(self):
        tosca_file = utils.get_template_path(
            "test_host_assignment.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_host_assignment.yaml")
        self._check_success(tosca_file, hot_file, {})

    def test_osc_translate_elk(self):
        tosca_file = utils.get_template_path(
            "tosca_elk.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_elk.yaml")
        params = {'github_url':
                  'http://github.com/paypal/rest-api-sample-app-nodejs.git',
                  'my_cpus': 4}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_nodejs_mongodb_two_instances(self):
        tosca_file = utils.get_template_path(
            "tosca_nodejs_mongodb_two_instances.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_nodejs_mongodb_two_instances.yaml")
        params = {'github_url':
                  'http://github.com/paypal/rest-api-sample-app-nodejs.git',
                  'my_cpus': 4}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_blockstorage_with_attachment(self):
        tosca_file = utils.get_template_path(
            "storage/tosca_blockstorage_with_attachment.yaml")
        hot_file = utils.get_template_path(
            "hot_output/storage/hot_blockstorage_with_attachment.yaml")
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '2000 MB',
                  'storage_snapshot_id': 'ssid'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_blockstorage_with_custom_relationship_type(self):
        tosca_file = utils.get_template_path(
            "storage/tosca_blockstorage_with_custom_relationship_type.yaml")
        hot_file = utils.get_template_path(
            "hot_output/storage/"
            "hot_blockstorage_with_custom_relationship_type.yaml")
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_blockstorage_with_relationship_template(self):
        tosca_file = utils.get_template_path(
            "storage/" +
            "tosca_blockstorage_with_relationship_template.yaml")
        hot_file = utils.get_template_path(
            "hot_output/storage/" +
            "hot_blockstorage_with_relationship_template.yaml")
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '1 GB'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_blockstorage_with_attachment_notation1(self):
        tosca_file = utils.get_template_path(
            "storage/" +
            "tosca_blockstorage_with_attachment_notation1.yaml")
        hot_file1 = utils.get_template_path(
            "hot_output/storage/" +
            "hot_blockstorage_with_attachment_notation1_alt1.yaml")
        hot_file2 = utils.get_template_path(
            "hot_output/storage/" +
            "hot_blockstorage_with_attachment_notation1_alt2.yaml")
        params = {'cpus': 1,
                  'storage_location': 'some_folder',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}
        try:
            self._check_success(tosca_file, hot_file1, params)
        except Exception:
            self._check_success(tosca_file, hot_file2, params)

    def test_osc_translate_blockstorage_with_attachment_notation2(self):
        tosca_file = utils.get_template_path(
            "storage/" +
            "tosca_blockstorage_with_attachment_notation2.yaml")
        hot_file1 = utils.get_template_path(
            "hot_output/storage/" +
            "hot_blockstorage_with_attachment_notation2_alt1.yaml")
        hot_file2 = utils.get_template_path(
            "hot_output/storage/" +
            "hot_blockstorage_with_attachment_notation2_alt2.yaml")
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}
        try:
            self._check_success(tosca_file, hot_file1, params)
        except Exception:
            self._check_success(tosca_file, hot_file2, params)

    def test_osc_translate_multiple_blockstorage_with_attachment(self):
        tosca_file = utils.get_template_path(
            "storage/" +
            "tosca_multiple_blockstorage_with_attachment.yaml")
        hot_file1 = utils.get_template_path(
            "hot_output/storage/" +
            "hot_multiple_blockstorage_with_attachment_alt1.yaml")
        hot_file2 = utils.get_template_path(
            "hot_output/storage/" +
            "hot_multiple_blockstorage_with_attachment_alt2.yaml")
        params = {'cpus': 1,
                  'storage_location': '/dev/vdc',
                  'storage_size': '1 GB',
                  'storage_snapshot_id': 'ssid'}
        try:
            self._check_success(tosca_file, hot_file1, params)
        except Exception:
            self._check_success(tosca_file, hot_file2, params)

    def test_osc_translate_single_object_store(self):
        tosca_file = utils.get_template_path(
            "storage/tosca_single_object_store.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_single_object_store.yaml")
        params = {'objectstore_name': 'myobjstore'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_one_server_one_network(self):
        tosca_file = utils.get_template_path(
            "network/tosca_one_server_one_network.yaml")
        hot_file = utils.get_template_path(
            "hot_output/network/" +
            "hot_one_server_one_network.yaml")
        params = {'network_name': 'private_net'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_server_on_existing_network(self):
        tosca_file = utils.get_template_path(
            "network/" +
            "tosca_server_on_existing_network.yaml")
        hot_file = utils.get_template_path(
            "hot_output/network/" +
            "hot_server_on_existing_network.yaml")
        params = {'network_name': 'private_net'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_two_servers_one_network(self):
        tosca_file = utils.get_template_path(
            "network/tosca_two_servers_one_network.yaml")
        hot_file = utils.get_template_path(
            "hot_output/network/" +
            "hot_two_servers_one_network.yaml")
        params = {'network_name': 'my_private_net',
                  'network_cidr': '10.0.0.0/24',
                  'network_start_ip': '10.0.0.100',
                  'network_end_ip': '10.0.0.150'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_one_server_three_networks(self):
        tosca_file = utils.get_template_path(
            "network/" +
            "tosca_one_server_three_networks.yaml")
        hot_file = utils.get_template_path(
            "hot_output/network/" +
            "hot_one_server_three_networks.yaml")
        self._check_success(tosca_file, hot_file, {})

    def test_osc_translate_software_component(self):
        tosca_file = utils.get_template_path("tosca_software_component.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_software_component.yaml")
        params = {'cpus': '1',
                  'download_url': 'http://www.software.com/download'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_web_application(self):
        tosca_file = utils.get_template_path("tosca_web_application.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_web_application.yaml")
        params = {'cpus': '2', 'context_root': 'my_web_app'}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_template_with_url_import(self):
        tosca_file = utils.get_template_path(
            "tosca_single_instance_wordpress_with_url_import.yaml")
        hot_file = utils.get_template_path(
            "hot_output/hot_single_instance_wordpress.yaml")
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_template_by_url_with_local_import(self):
        tosca_file = ("https://raw.githubusercontent.com/openstack/" +
                      "heat-translator/master/translator/tests/data/" +
                      "tosca_single_instance_wordpress.yaml")
        hot_file = utils.get_template_path(
            "hot_output/" +
            "hot_single_instance_wordpress.yaml")
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_template_by_url_with_local_abspath_import(self):
        tosca_file = ("https://raw.githubusercontent.com/openstack/" +
                      "heat-translator/master/translator/tests/data/" +
                      "tosca_single_instance_wordpress_with_local_abspath" +
                      "_import.yaml")
        hot_file = utils.get_template_path(
            "hot_output/" +
            "hot_single_instance_wordpress.yaml")
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}

        expected_msg = _('Absolute file name "/tmp/wordpress.yaml" cannot be '
                         'used in a URL-based input template "https://raw.'
                         'githubusercontent.com/openstack/heat-translator/'
                         'master/translator/tests/data/tosca_single_instance_'
                         'wordpress_with_local_abspath_import.yaml".')
        self._check_error(tosca_file, hot_file, params, ValidationError,
                          expected_msg, ImportError)

    def test_osc_translate_template_by_url_with_url_import(self):
        tosca_url = ("https://raw.githubusercontent.com/openstack/" +
                     "heat-translator/master/translator/tests/data/" +
                     "tosca_single_instance_wordpress_with_url_import.yaml")
        hot_file = utils.get_template_path(
            "hot_output/" +
            "hot_single_instance_wordpress.yaml")
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._check_success(tosca_url, hot_file, params)

    def test_osc_translate_hello_world_csar(self):
        tosca_file = utils.get_template_path("csar_hello_world.zip")
        hot_file = utils.get_template_path(
            "hot_output/hot_hello_world.yaml")
        self._check_success(tosca_file, hot_file, {})

    def test_osc_single_instance_wordpress_csar(self):
        tosca_file = utils.get_template_path(
            "csar_single_instance_wordpress.zip")
        hot_file = utils.get_template_path(
            "hot_output/" +
            "hot_single_instance_wordpress_from_csar.yaml")
        params = {'db_name': 'wordpress',
                  'db_user': 'wp_user',
                  'db_pwd': 'wp_pass',
                  'db_root_pwd': 'passw0rd',
                  'db_port': 3366,
                  'cpus': 8}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_elk_csar_from_url(self):
        tosca_file = ("https://github.com/openstack/heat-translator/raw/" +
                      "master/translator/tests/data/csar_elk.zip")
        hot_file = utils.get_template_path(
            "hot_output/hot_elk_from_csar.yaml")
        params = {'github_url':
                  'http://github.com/paypal/rest-api-sample-app-nodejs.git',
                  'my_cpus': 4}
        self._check_success(tosca_file, hot_file, params)

    def test_osc_translate_csar_not_zip(self):
        tosca_file = utils.get_template_path("csar_not_zip.zip")
        hot_file = ''
        expected_msg = _('"%s" is not a valid zip file.') % tosca_file
        self._check_error(tosca_file, hot_file, {}, ValidationError,
                          expected_msg, ValidationError)

    def test_osc_translate_csar_metadata_not_yaml(self):
        tosca_file = utils.get_template_path("csar_metadata_not_yaml.zip")
        hot_file = ''
        expected_msg = _('The file "TOSCA-Metadata/TOSCA.meta" in the CSAR '
                         '"%s" does not contain valid YAML'
                         ' content.') % tosca_file
        self._check_error(tosca_file, hot_file, {}, ValidationError,
                          expected_msg, ValidationError)

    def test_osc_translate_csar_wrong_metadata_file(self):
        tosca_file = utils.get_template_path("csar_wrong_metadata_file.zip")
        hot_file = ''

        expected_msg = _('"%s" is not a valid CSAR as it does not contain the '
                         'required file "TOSCA.meta" in the folder '
                         '"TOSCA-Metadata".') % tosca_file
        self._check_error(tosca_file, hot_file, {}, ValidationError,
                          expected_msg, ValidationError)

    def test_osc_translate_csar_wordpress_invalid_import_path(self):
        tosca_file = utils.get_template_path(
            "csar_wordpress_invalid_import_path.zip")
        hot_file = ''
        expected_msg = _('Import '
                         '"Invalid_import_path/wordpress.yaml" is not valid.')
        self._check_error(tosca_file, hot_file, {}, ValidationError,
                          expected_msg, ImportError)

    def test_osc_translate_csar_wordpress_invalid_script_url(self):
        tosca_file = utils.get_template_path(
            "csar_wordpress_invalid_script_url.zip")
        hot_file = ''
        expected_msg = _('The resource at '
                         '"https://raw.githubusercontent.com/openstack/'
                         'heat-translator/master/translator/tests/data/'
                         'custom_types/wordpress1.yaml" cannot be accessed.')
        self._check_error(tosca_file, hot_file, {}, ValidationError,
                          expected_msg, URLException)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import OrderedDict

from toscaparser.tests.base import TestCase
from translator.hot.syntax.hot_parameter import CONSTRAINTS
from translator.hot.syntax.hot_parameter import DEFAULT
from translator.hot.syntax.hot_parameter import DESCRIPTION
from translator.hot.syntax.hot_parameter import HIDDEN
from translator.hot.syntax.hot_parameter import HotParameter
from translator.hot.syntax.hot_parameter import LABEL
from translator.hot.syntax.hot_parameter import TYPE

TEST_CONSTRAINTS = {'equal': 'allowed_values', 'greater_than': 'range'}


class HotParameterTest(TestCase):

    # This test ensures the variables set during the creation of a HotParameter
    # object are returned in an OrderedDict when calling get_dict_output().
    def test_dict_output(self):
        name = 'HotParameterTest'
        hot_parameter = HotParameter(name, 'Type',
                                     label='Label',
                                     description='Description',
                                     default='Default',
                                     hidden=True,
                                     constraints=TEST_CONSTRAINTS)
        expected_dict = OrderedDict([(TYPE, 'Type'), (LABEL, 'Label'),
                                     (DESCRIPTION, 'Description'),
                                     (DEFAULT, 'Default'), (HIDDEN, True),
                                     (CONSTRAINTS, TEST_CONSTRAINTS)])

        self.assertEqual(hot_parameter.get_dict_output()[name], expected_dict)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from toscaparser.nodetemplate import NodeTemplate
from toscaparser.tests.base import TestCase
import toscaparser.utils.yamlparser
from translator.hot.tosca.tosca_compute import ToscaCompute


class ToscaComputeTest(TestCase):

    def _tosca_compute_test(self, tpl_snippet, expectedprops):
        nodetemplates = (toscaparser.utils.yamlparser.
                         simple_parse(tpl_snippet)['node_templates'])
        name = list(nodetemplates.keys())[0]
        nodetemplate = NodeTemplate(name, nodetemplates)
        nodetemplate.validate()
        toscacompute = ToscaCompute(nodetemplate)
        toscacompute.handle_properties()

        self.assertEqual(expectedprops, toscacompute.properties)

    def test_node_compute_with_host_and_os_capabilities(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              host:
                properties:
                  disk_size: 10 GB
                  num_cpus: 4
                  mem_size: 4 GB
              os:
                properties:
                  architecture: x86_64
                  type: Linux
                  distribution: Fedora
                  version: 18.0
        '''
        expectedprops = {'flavor': 'm1.large',
                         'image': 'fedora-amd64-heat-config',
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    def test_node_compute_without_os_capabilities(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              host:
                properties:
                  disk_size: 10 GB
                  num_cpus: 4
                  mem_size: 4 GB
              #left intentionally
        '''
        expectedprops = {'flavor': 'm1.large',
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    def test_node_compute_without_host_capabilities(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              os:
                properties:
                  architecture: x86_64
                  type: Linux
                  distribution: Fedora
                  version: 18.0
        '''
        expectedprops = {'flavor': None,
                         'image': 'fedora-amd64-heat-config',
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    def test_node_compute_without_properties_and_os_capabilities(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            properties:
              #left intentionally
            capabilities:
              #left intentionally
        '''
        expectedprops = {'flavor': None,
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    def test_node_compute_with_only_type(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
        '''
        expectedprops = {'flavor': None,
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    def test_node_compute_host_capabilities_without_properties(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              host:
                properties:
                #left intentionally
        '''
        expectedprops = {'flavor': None,
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    def test_node_compute_host_capabilities_without_disk_size(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              host:
                properties:
                  num_cpus: 4
                  mem_size: 4 GB
        '''
        expectedprops = {'flavor': 'm1.large',
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    def test_node_compute_host_capabilities_without_mem_size(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              host:
                properties:
                  num_cpus: 4
                  disk_size: 10 GB
        '''
        expectedprops = {'flavor': 'm1.large',
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    def test_node_compute_host_capabilities_without_mem_size_disk_size(self):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              host:
                properties:
                  num_cpus: 4
        '''
        expectedprops = {'flavor': 'm1.large',
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(
            tpl_snippet,
            expectedprops)

    @mock.patch('translator.common.flavors.get_flavors')
    def test_node_compute_with_nova_flavor(self, mock_flavor):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              host:
                properties:
                  num_cpus: 1
                  disk_size: 1 GB
                  mem_size: 1 GB
        '''
        mock_flavor.return_value = {
            'm1.mock_flavor': {
                'mem_size': 1024,
                'disk_size': 1,
                'num_cpus': 1}
        }
        expectedprops = {'flavor': 'm1.mock_flavor',
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(tpl_snippet, expectedprops)

    @mock.patch('translator.common.images.get_images')
    def test_node_compute_with_glance_image(self, mock_images):
        tpl_snippet = '''
        node_templates:
          server:
            type: tosca.nodes.Compute
            capabilities:
              host:
                properties:
                  num_cpus: 1
                  disk_size: 1 GB
                  mem_size: 1 GB
              os:
                properties:
                  architecture: x86_64
                  type: Linux
                  distribution: Fake Distribution
                  version: 19.0
        '''
        mock_images.return_value = {
            'fake-image-foobar': {'architecture': 'x86_64',
                                  'type': 'Linux',
                                  'distribution': 'Fake Distribution',
                                  'version': '19.0'},
            'fake-image-foobar-old': {'architecture': 'x86_64',
                                      'type': 'Linux',
                                      'distribution': 'Fake Distribution',
                                      'version': '18.0'}
        }
        expectedprops = {'flavor': 'm1.small',
                         'image': 'fake-image-foobar',
                         'user_data_format': 'SOFTWARE_CONFIG'}
        self._tosca_compute_test(tpl_snippet, expectedprops)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from translator.hot.syntax.hot_resource import HotResource
# Name used to dynamically load appropriate map class.
TARGET_CLASS_NAME = 'ToscaAutoscaling'
ALARM_STATISTIC = {'mean': 'mean', 'median': 'median', 'summary': 'sum',
                   'maximum': 'max', 'minimum': 'min', 'last': 'last',
                   'std': 'std', 'first': 'first', 'count': 'count'}
SCALING_RESOURCES = ["OS::Heat::ScalingPolicy", "OS::Heat::AutoScalingGroup",
                     "OS::Aodh::GnocchiAggregationByResourcesAlarm",
                     "OS::Aodh::EventAlarm"]


class ToscaAutoscaling(HotResource):
    '''Translate TOSCA node type tosca.policies.Scaling'''

    toscatype = 'tosca.policies.Scaling'

    def __init__(self, policy, csar_dir=None, hot_template_parameters=None):
        hot_type = "OS::Heat::ScalingPolicy"
        super(ToscaAutoscaling, self).__init__(policy,
                                               type=hot_type,
                                               csar_dir=csar_dir)
        self.policy = policy
        self.hot_template_parameters = hot_template_parameters

    def handle_expansion(self):
        if self.policy.entity_tpl.get('triggers'):
            hot_type = 'OS::Aodh::GnocchiAggregationByResourcesAlarm'
            sample = self.policy.\
                entity_tpl["triggers"]["resize_compute"]["condition"]
            prop = {}
            prop["description"] = self.policy.entity_tpl.get('description')
            prop["metric"] = "cpu_util"
            if sample:
                prop["aggregation_method"] = \
                    ALARM_STATISTIC[sample["aggregation_method"]]
                prop["granularity"] = sample["granularity"]
                prop["threshold"] = sample["evaluations"]
                prop["resource_type"] = sample.get("resource_type", "instance")
            prop["comparison_operator"] = "gt"
            alarm_name = self.name.replace('_scale_in', '').\
                replace('_scale_out', '')
            ceilometer_resources = HotResource(self.nodetemplate,
                                               type=hot_type,
                                               name=alarm_name + '_alarm',
                                               properties=prop)
            hot_resources = [ceilometer_resources]
            return hot_resources

    def handle_properties(self, resources):
        self.properties = {}
        self.properties["auto_scaling_group_id"] = {
            'get_resource': self.policy.name + '_group'
        }
        self.properties["adjustment_type"] = "change_in_capacity "
        self.properties["scaling_adjustment"] = self.\
            policy.entity_tpl["properties"]["increment"]
        self.properties["cooldown"] =\
            self.policy.entity_tpl["properties"]["cooldown"]
        delete_res_names = []
        scale_res = []
        for index, resource in enumerate(resources):
            if resource.name in self.policy.targets and \
                resource.type != 'OS::Heat::AutoScalingGroup':
                temp = self.policy.entity_tpl["properties"]
                props = {}
                res = {}
                res["min_size"] = temp["min_instances"]
                res["max_size"] = temp["max_instances"]
                res["desired_capacity"] = temp["default_instances"]
                res["cooldown"] = temp["cooldown"]
                props['type'] = resource.type
                props['properties'] = resource.properties
                res['resource'] = {'type': self.policy.name + '_res.yaml'}
                scaling_resources = \
                    HotResource(resource,
                                type='OS::Heat::AutoScalingGroup',
                                name=self.policy.name + '_group',
                                properties=res)

            if resource.type not in SCALING_RESOURCES:
                delete_res_names.append(resource.name)
                scale_res.append(resource)
        yaml_name = self.policy.name + '_res.yaml'
        self.nested_template = self._handle_nested_template(
            scale_res,
            yaml_name,
            self.hot_template_parameters)
        resources = [tmp_res
                     for tmp_res in resources
                     if tmp_res.name not in delete_res_names]
        resources.append(scaling_resources)
        return resources

    def extract_substack_templates(self, base_filename, hot_template_version):
        return self.nested_template

    def embed_substack_templates(self, hot_template_version):
        pass



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from translator.hot.syntax.hot_resource import HotResource

# Name used to dynamically load appropriate map class.
TARGET_CLASS_NAME = 'ToscaPolicies'


class ToscaPolicies(HotResource):
    '''Translate TOSCA policy type tosca.policies.Placement.'''

    toscatype = 'tosca.policies.Placement'

    def __init__(self, policy, csar_dir=None):
        super(ToscaPolicies, self).__init__(policy,
                                            type='OS::Nova::ServerGroup',
                                            csar_dir=csar_dir)
        self.policy = policy

    def handle_properties(self, resources):
        group_policy = "%(soft-prefix)s%(policy)s" % {
            "soft-prefix": "soft-" if not self.get_tosca_props().get(
                "strict", True) else "",
            "policy": self.get_tosca_props().get("policy", "affinity"),
        }
        self.properties["name"] = self.name
        self.properties["policies"] = [group_policy]
        for resource in resources:
            if resource.name in self.policy.targets:
                if not resource.properties.get("scheduler_hints"):
                    resource.properties["scheduler_hints"] = {}
                resource.properties["scheduler_hints"].update(
                    {"group": {"get_resource": self.name}})



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class HotOutput(object):
    '''Attributes for HOT output section.'''

    def __init__(self, name, value, description=None):
        self.name = name
        self.value = value
        self.description = description

    def get_dict_output(self):
        if self.description:
            return {self.name: {'value': self.value,
                                'description': self.description}}
        else:
            return {self.name: {'value': self.value}}



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from toscaparser.utils.gettextutils import _
from translator.hot.syntax.hot_output import HotOutput

log = logging.getLogger('heat-translator')


class TranslateOutputs(object):
    '''Translate TOSCA Outputs to Heat Outputs.'''

    def __init__(self, outputs, node_translator):
        log.debug(_('Translating TOSCA outputs to HOT outputs.'))
        self.outputs = outputs
        self.nodes = node_translator

    def translate(self):
        return self._translate_outputs()

    def _translate_outputs(self):
        hot_outputs = []
        for output in self.outputs:
            hot_value = self.nodes.translate_param_value(output.value, None)
            if hot_value is not None:
                hot_outputs.append(HotOutput(output.name, hot_value,
                                             output.description))
        return hot_outputs



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from toscaparser.utils.gettextutils import _
from translator.common import flavors as nova_flavors
from translator.common import images as glance_images
import translator.common.utils
from translator.hot.syntax.hot_resource import HotResource


log = logging.getLogger('heat-translator')


# Name used to dynamically load appropriate map class.
TARGET_CLASS_NAME = 'ToscaCompute'


class ToscaCompute(HotResource):
    """Translate TOSCA node type tosca.nodes.Compute."""

    COMPUTE_HOST_PROP = (DISK_SIZE, MEM_SIZE, NUM_CPUS) = \
                        ('disk_size', 'mem_size', 'num_cpus')

    COMPUTE_OS_PROP = (ARCHITECTURE, DISTRIBUTION, TYPE, VERSION) = \
                      ('architecture', 'distribution', 'type', 'version')

    IMAGE_OS_PROP = (OS_DISTRO, OS_TYPE, OS_VERSION) = \
                    ('os_distro', 'os_type', 'os_version')
    toscatype = 'tosca.nodes.Compute'

    ALLOWED_NOVA_SERVER_PROPS = \
        ('admin_pass', 'availability_zone', 'block_device_mapping',
         'block_device_mapping_v2', 'config_drive', 'diskConfig', 'flavor',
         'flavor_update_policy', 'image', 'image_update_policy', 'key_name',
         'metadata', 'name', 'networks', 'personality', 'reservation_id',
         'scheduler_hints', 'security_groups', 'software_config_transport',
         'user_data', 'user_data_format', 'user_data_update_policy')

    def __init__(self, nodetemplate, csar_dir=None):
        super(ToscaCompute, self).__init__(nodetemplate,
                                           type='OS::Nova::Server',
                                           csar_dir=csar_dir)
        # List with associated hot port resources with this server
        self.assoc_port_resources = []
        pass

    def handle_properties(self):
        self.properties = self.translate_compute_flavor_and_image(
            self.nodetemplate.get_capability('host'),
            self.nodetemplate.get_capability('os'))
        self.properties['user_data_format'] = 'SOFTWARE_CONFIG'
        tosca_props = self.get_tosca_props()
        for key, value in tosca_props.items():
            if key in self.ALLOWED_NOVA_SERVER_PROPS:
                self.properties[key] = value

    # To be reorganized later based on new development in Glance and Graffiti
    def translate_compute_flavor_and_image(self,
                                           host_capability,
                                           os_capability):
        hot_properties = {}
        host_cap_props = {}
        os_cap_props = {}
        image = None
        flavor = None
        if host_capability:
            for prop in host_capability.get_properties_objects():
                host_cap_props[prop.name] = prop.value
            # if HOST properties are not specified, we should not attempt to
            # find best match of flavor
            if host_cap_props:
                flavor = self._best_flavor(host_cap_props)
        if os_capability:
            for prop in os_capability.get_properties_objects():
                os_cap_props[prop.name] = prop.value
            # if OS properties are not specified, we should not attempt to
            # find best match of image
            if os_cap_props:
                image = self._best_image(os_cap_props)
        hot_properties['flavor'] = flavor
        if image:
            hot_properties['image'] = image
        else:
            hot_properties.pop('image', None)
        return hot_properties

    def _best_flavor(self, properties):
        log.info(_('Choosing the best flavor for given attributes.'))
        # Check whether user exported all required environment variables.
        flavors = nova_flavors.get_flavors()

        # start with all flavors
        match_all = flavors.keys()

        # TODO(anyone): Handle the case where the value contains something like
        # get_input instead of a value.
        # flavors that fit the CPU count
        cpu = properties.get(self.NUM_CPUS)
        if cpu is None:
            self._log_compute_msg(self.NUM_CPUS, 'flavor')
        match_cpu = self._match_flavors(match_all, flavors, self.NUM_CPUS, cpu)

        # flavors that fit the mem size
        mem = properties.get(self.MEM_SIZE)
        if mem:
            mem = translator.common.utils.MemoryUnit.convert_unit_size_to_num(
                mem, 'MB')
        else:
            self._log_compute_msg(self.MEM_SIZE, 'flavor')
        match_cpu_mem = self._match_flavors(match_cpu, flavors,
                                            self.MEM_SIZE, mem)
        # flavors that fit the disk size
        disk = properties.get(self.DISK_SIZE)
        if disk:
            disk = translator.common.utils.MemoryUnit.\
                convert_unit_size_to_num(disk, 'GB')
        else:
            self._log_compute_msg(self.DISK_SIZE, 'flavor')
        match_cpu_mem_disk = self._match_flavors(match_cpu_mem, flavors,
                                                 self.DISK_SIZE, disk)
        # if multiple match, pick the flavor with the least memory
        # the selection can be based on other heuristic, e.g. pick one with the
        # least total resource
        if len(match_cpu_mem_disk) > 1:
            return self._least_flavor(match_cpu_mem_disk, flavors, 'mem_size')
        elif len(match_cpu_mem_disk) == 1:
            return match_cpu_mem_disk[0]
        else:
            return None

    def _best_image(self, properties):
        if 'image' in properties:
            return properties['image']

        # Check whether user exported all required environment variables.
        images = glance_images.get_images()
        match_all = images.keys()

        architecture = properties.get(self.ARCHITECTURE)
        if architecture is None:
            self._log_compute_msg(self.ARCHITECTURE, 'image')
        match_arch = self._match_images(match_all, images,
                                        [self.ARCHITECTURE], architecture)

        image_type = properties.get(self.TYPE)
        if image_type is None:
            self._log_compute_msg(self.TYPE, 'image')
        match_type = self._match_images(match_arch, images, [self.TYPE,
                                                             self.OS_TYPE],
                                        image_type)

        distribution = properties.get(self.DISTRIBUTION)
        if distribution is None:
            self._log_compute_msg(self.DISTRIBUTION, 'image')
        match_distribution = self._match_images(match_type, images,
                                                [self.DISTRIBUTION,
                                                 self.OS_DISTRO],
                                                distribution)

        version = properties.get(self.VERSION)
        if version is None:
            self._log_compute_msg(self.VERSION, 'image')
        match_version = self._match_images(match_distribution, images,
                                           [self.VERSION, self.OS_VERSION],
                                           version)

        if len(match_version):
            return list(match_version)[0]

    @staticmethod
    def _match_flavors(this_list, this_dict, attr, size):
        """Return from this list all flavors matching the attribute size."""
        if not size:
            return list(this_list)
        matching_flavors = []
        for flavor in this_list:
            if isinstance(size, int):
                if this_dict[flavor][attr] >= size:
                    matching_flavors.append(flavor)
        log.debug(_('Returning list of flavors matching the attribute size.'))
        return matching_flavors

    @staticmethod
    def _least_flavor(this_list, this_dict, attr):
        """Return from this list the flavor with the smallest attr."""
        least_flavor = this_list[0]
        for flavor in this_list:
            if this_dict[flavor][attr] < this_dict[least_flavor][attr]:
                least_flavor = flavor
        return least_flavor

    @staticmethod
    def _match_images(this_list, this_dict, attr_list, prop):
        if not prop:
            return this_list
        matching_images = []
        for image in this_list:
            for attr in attr_list:
                if attr in this_dict[image]:
                    if this_dict[image][attr].lower() == str(prop).lower():
                        matching_images.insert(0, image)
                else:
                    matching_images.append(image)
        return matching_images

    def get_hot_attribute(self, attribute, args):
        attr = {}
        # Convert from a TOSCA attribute for a nodetemplate to a HOT
        # attribute for the matching resource.  Unless there is additional
        # runtime support, this should be a one to one mapping.

        # Note: We treat private and public IP  addresses equally, but
        # this will change in the future when TOSCA starts to support
        # multiple private/public IP addresses.
        log.debug(_('Converting TOSCA attribute for a nodetemplate to a HOT \
                  attriute.'))
        if (attribute == 'private_address' or
                attribute == 'public_address'):
            attr['get_attr'] = [self.name, 'networks', 'private', 0]

        return attr

    @staticmethod
    def _log_compute_msg(prop, what):
        msg = _('No value is provided for Compute capability '
                'property "%(prop)s". This may set an undesired "%(what)s" '
                'in the template.') % {'prop': prop, 'what': what}
        log.warning(msg)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from collections import OrderedDict
import logging
import os
import textwrap
from toscaparser.utils.gettextutils import _
import yaml

log = logging.getLogger('heat-translator')


class HotTemplate(object):
    '''Container for full Heat Orchestration template.'''

    SECTIONS = (VERSION, DESCRIPTION, PARAMETER_GROUPS, PARAMETERS,
                RESOURCES, OUTPUTS, MAPPINGS) = \
               ('heat_template_version', 'description', 'parameter_groups',
                'parameters', 'resources', 'outputs', '__undefined__')

    VERSIONS = (LATEST,) = ('2013-05-23',)

    def __init__(self):
        self.resources = []
        self.outputs = []
        self.parameters = []
        self.description = ""

    def represent_ordereddict(self, dumper, data):
        nodes = []
        for key, value in data.items():
            node_key = dumper.represent_data(key)
            node_value = dumper.represent_data(value)
            nodes.append((node_key, node_value))
        return yaml.nodes.MappingNode('tag:yaml.org,2002:map', nodes)

    def output_to_yaml_files_dict(self, base_filename,
                                  hot_template_version=LATEST):
        yaml_files_dict = {}
        base_filename, ext = os.path.splitext(base_filename)

        # convert from inlined substack to a substack defined in another file
        for resource in self.resources:
            yaml_files_dict.update(
                resource.extract_substack_templates(base_filename,
                                                    hot_template_version))

        yaml_files_dict[base_filename + ext] = \
            self.output_to_yaml(hot_template_version, False)

        return yaml_files_dict

    def output_to_yaml(self, hot_template_version=LATEST,
                       embed_substack_templates=True):
        log.debug(_('Converting translated output to yaml format.'))

        if embed_substack_templates:
            # fully inlined substack by storing the template as a blob string
            for resource in self.resources:
                resource.embed_substack_templates(hot_template_version)

        dict_output = OrderedDict()
        # Version
        version_string = self.VERSION + ": " + hot_template_version + "\n\n"

        # Description
        desc_str = ""
        if self.description:
            # Wrap the text to a new line if the line exceeds 80 characters.
            wrapped_txt = "\n  ".join(textwrap.wrap(self.description, 80))
            desc_str = self.DESCRIPTION + ": >\n  " + wrapped_txt + "\n\n"

        # Parameters
        all_params = OrderedDict()
        for parameter in self.parameters:
            all_params.update(parameter.get_dict_output())
        dict_output.update({self.PARAMETERS: all_params})

        # Resources
        all_resources = OrderedDict()
        for resource in self.resources:
            if not resource.hide_resource:
                all_resources.update(resource.get_dict_output())
        dict_output.update({self.RESOURCES: all_resources})

        # Outputs
        all_outputs = OrderedDict()
        for output in self.outputs:
            all_outputs.update(output.get_dict_output())
        dict_output.update({self.OUTPUTS: all_outputs})

        yaml.add_representer(OrderedDict, self.represent_ordereddict)
        yaml.add_representer(dict, self.represent_ordereddict)
        yaml_string = yaml.dump(dict_output, default_flow_style=False)
        # get rid of the '' from yaml.dump around numbers
        # also replace double return lines with a single one
        # seems to be a bug in the serialization of multiline literal scalars
        yaml_string = yaml_string.replace('\'', '') .replace('\n\n', '\n')
        return version_string + desc_str + yaml_string



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
from toscaparser.dataentity import DataEntity
from toscaparser.elements.scalarunit import ScalarUnit_Size
from toscaparser.parameters import Input
from toscaparser.utils.gettextutils import _
from toscaparser.utils.validateutils import TOSCAVersionProperty
from translator.hot.syntax.hot_parameter import HotParameter


INPUT_CONSTRAINTS = (CONSTRAINTS, DESCRIPTION, LENGTH, RANGE,
                     MIN, MAX, ALLOWED_VALUES, ALLOWED_PATTERN) = \
                    ('constraints', 'description', 'length', 'range',
                     'min', 'max', 'allowed_values', 'allowed_pattern')

TOSCA_CONSTRAINT_OPERATORS = (EQUAL, GREATER_THAN, GREATER_OR_EQUAL, LESS_THAN,
                              LESS_OR_EQUAL, IN_RANGE, VALID_VALUES, LENGTH,
                              MIN_LENGTH, MAX_LENGTH, PATTERN) = \
                             ('equal', 'greater_than', 'greater_or_equal',
                              'less_than', 'less_or_equal', 'in_range',
                              'valid_values', 'length', 'min_length',
                              'max_length', 'pattern')

TOSCA_TO_HOT_CONSTRAINTS_ATTRS = {'equal': 'allowed_values',
                                  'greater_than': 'range',
                                  'greater_or_equal': 'range',
                                  'less_than': 'range',
                                  'less_or_equal': 'range',
                                  'in_range': 'range',
                                  'valid_values': 'allowed_values',
                                  'length': 'length',
                                  'min_length': 'length',
                                  'max_length': 'length',
                                  'pattern': 'allowed_pattern'}

TOSCA_TO_HOT_INPUT_TYPES = {'string': 'string',
                            'integer': 'number',
                            'float': 'number',
                            'boolean': 'boolean',
                            'timestamp': 'string',
                            'scalar-unit.size': 'number',
                            'version': 'string',
                            'null': 'string',
                            'PortDef': 'number',
                            'list': 'list'}

log = logging.getLogger('heat-translator')


class TranslateInputs(object):

    '''Translate TOSCA Inputs to Heat Parameters.'''

    def __init__(self, inputs, parsed_params, deploy=None):
        self.inputs = inputs
        self.parsed_params = parsed_params
        self.deploy = deploy

    def translate(self):
        return self._translate_inputs()

    def _translate_inputs(self):
        hot_inputs = []
        if 'key_name' in self.parsed_params and 'key_name' not in self.inputs:
            name = 'key_name'
            type = 'string'
            default = self.parsed_params[name]
            schema_dict = {'type': type, 'default': default}
            input = Input(name, schema_dict)
            self.inputs.append(input)

        log.info(_('Translating TOSCA input type to HOT input type.'))
        for input in self.inputs:
            hot_default = None
            hot_input_type = TOSCA_TO_HOT_INPUT_TYPES[input.type]

            if input.name in self.parsed_params:
                hot_default = DataEntity.validate_datatype(
                    input.type, self.parsed_params[input.name])
            elif input.default is not None:
                hot_default = DataEntity.validate_datatype(input.type,
                                                           input.default)
            else:
                if self.deploy:
                    msg = _("Need to specify a value "
                            "for input {0}.").format(input.name)
                    log.error(msg)
                    raise Exception(msg)
            if input.type == "scalar-unit.size":
                # Assumption here is to use this scalar-unit.size for size of
                # cinder volume in heat templates and will be in GB.
                # should add logic to support other types if needed.
                input_value = hot_default
                hot_default = (ScalarUnit_Size(hot_default).
                               get_num_from_scalar_unit('GiB'))
                if hot_default == 0:
                    msg = _('Unit value should be > 0.')
                    log.error(msg)
                    raise Exception(msg)
                elif int(hot_default) < hot_default:
                    hot_default = int(hot_default) + 1
                    log.warning(_("Cinder unit value should be in multiples"
                                  " of GBs. So corrected %(input_value)s "
                                  "to %(hot_default)s GB.")
                                % {'input_value': input_value,
                                   'hot_default': hot_default})
            if input.type == 'version':
                hot_default = TOSCAVersionProperty(hot_default).get_version()

            hot_constraints = []
            if input.constraints:
                for constraint in input.constraints:
                    if hot_default:
                        constraint.validate(hot_default)
                    hc, hvalue = self._translate_constraints(
                        constraint.constraint_key, constraint.constraint_value)
                    hot_constraints.append({hc: hvalue})

            hot_inputs.append(HotParameter(name=input.name,
                                           type=hot_input_type,
                                           description=input.description,
                                           default=hot_default,
                                           constraints=hot_constraints))
        return hot_inputs

    def _translate_constraints(self, name, value):
        hot_constraint = TOSCA_TO_HOT_CONSTRAINTS_ATTRS[name]

        # Offset used to support less_than and greater_than.
        # TODO(anyone):  when parser supports float, verify this works
        offset = 1

        if name == EQUAL:
            hot_value = [value]
        elif name == GREATER_THAN:
            hot_value = {"min": value + offset}
        elif name == GREATER_OR_EQUAL:
            hot_value = {"min": value}
        elif name == LESS_THAN:
            hot_value = {"max": value - offset}
        elif name == LESS_OR_EQUAL:
            hot_value = {"max": value}
        elif name == IN_RANGE:
            # value is list type here
            min_value = min(value)
            max_value = max(value)
            hot_value = {"min": min_value, "max": max_value}
        elif name == LENGTH:
            hot_value = {"min": value, "max": value}
        elif name == MIN_LENGTH:
            hot_value = {"min": value}
        elif name == MAX_LENGTH:
            hot_value = {"max": value}
        else:
            hot_value = value
        return hot_constraint, hot_value



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import argparse
import codecs
import logging
import logging.config
import os
import sys
import uuid
import yaml
import zipfile

# NOTE(aloga): As per upstream developers requirement this needs to work
# without the clients, therefore we need to pass if we cannot import them
try:
    from keystoneauth1 import loading
except ImportError:
    keystone_client_avail = False
else:
    keystone_client_avail = True

try:
    import heatclient.client
except ImportError:
    heat_client_avail = False
else:
    heat_client_avail = True


from toscaparser.tosca_template import ToscaTemplate
from toscaparser.utils.gettextutils import _
from toscaparser.utils.urlutils import UrlUtils
from translator.common import flavors
from translator.common import images
from translator.common import utils
from translator.conf.config import ConfigProvider
from translator.hot.tosca_translator import TOSCATranslator

"""
Test the heat-translator translation from command line as:
#heat-translator
  --template-file=<path to the YAML template>
  --template-type=<type of template e.g. tosca>
  --parameters="purpose=test"
Takes three user arguments,
1. Path to the file that needs to be translated (required)
2. type of translation (e.g. tosca) (optional)
3. Input parameters (optional)

In order to use heat-translator to only validate template,
without actual translation, pass --validate-only=true along with
other required arguments.

"""
conf_file = ConfigProvider.get_translator_logging_file()
try:
    logging.config.fileConfig(conf_file)
except Exception:
    pass
log = logging.getLogger("heat-translator")


class TranslatorShell(object):

    SUPPORTED_TYPES = ['tosca']
    TOSCA_CSAR_META_DIR = "TOSCA-Metadata"

    def get_parser(self, argv):
        parser = argparse.ArgumentParser(prog="heat-translator")

        parser.add_argument('--template-file',
                            metavar='<filename>',
                            required=True,
                            help=_('Template file to load.'))

        parser.add_argument('--output-file',
                            metavar='<filename>',
                            help=_('Where to store the output file. If not '
                                   'passed, it will be printed to stdin.'))

        parser.add_argument('--template-type',
                            metavar='<input-template-type>',
                            choices=self.SUPPORTED_TYPES,
                            default='tosca',
                            help=(_('Template type to parse. Choose between '
                                    '%s.') % self.SUPPORTED_TYPES))

        parser.add_argument('--parameters',
                            metavar='<param1=val1;param2=val2;...>',
                            help=_('Optional input parameters.'))

        parser.add_argument('--validate-only',
                            action='store_true',
                            default=False,
                            help=_('Only validate input template, do not '
                                   'perform translation.'))

        parser.add_argument('--deploy',
                            action='store_true',
                            default=False,
                            help=_('Whether to deploy the generated template '
                                   'or not.'))

        parser.add_argument('--stack-name',
                            metavar='<stack-name>',
                            required=False,
                            help=_('The name to use for the Heat stack when '
                                   'deploy the generated template.'))

        self._append_global_identity_args(parser, argv)

        return parser

    def _append_global_identity_args(self, parser, argv):
        if not keystone_client_avail:
            return

        loading.register_session_argparse_arguments(parser)

        default_auth_plugin = 'password'
        if 'os-token' in argv:
            default_auth_plugin = 'token'
        loading.register_auth_argparse_arguments(
            parser, argv, default=default_auth_plugin)

    def main(self, argv):

        parser = self.get_parser(argv)
        (args, args_list) = parser.parse_known_args(argv)

        template_file = args.template_file
        template_type = args.template_type
        output_file = args.output_file
        validate_only = args.validate_only
        deploy = args.deploy
        stack_name = args.stack_name

        parsed_params = {}
        if args.parameters:
            parsed_params = self._parse_parameters(args.parameters)

        a_file = os.path.isfile(template_file)
        a_url = UrlUtils.validate_url(template_file) if not a_file else False
        if a_file or a_url:
            if validate_only:
                ToscaTemplate(template_file, parsed_params, a_file)
                msg = (_('The input "%(template_file)s" successfully passed '
                         'validation.') % {'template_file': template_file})
                print(msg)
            else:
                if keystone_client_avail:
                    try:
                        keystone_auth = (
                            loading.load_auth_from_argparse_arguments(args)
                        )
                        keystone_session = (
                            loading.load_session_from_argparse_arguments(
                                args,
                                auth=keystone_auth
                            )
                        )
                        images.SESSION = keystone_session
                        flavors.SESSION = keystone_session
                    except Exception:
                        keystone_session = None

                translator = self._get_translator(template_type,
                                                  template_file,
                                                  parsed_params, a_file,
                                                  deploy)

                if translator and deploy:
                    if not keystone_client_avail or not heat_client_avail:
                        raise RuntimeError(_('Could not find Heat or Keystone'
                                             'client to deploy, aborting '))
                    if not keystone_session:
                        raise RuntimeError(_('Impossible to login with '
                                             'Keystone to deploy on Heat, '
                                             'please check your credentials'))

                    file_name = os.path.basename(
                        os.path.splitext(template_file)[0])
                    self.deploy_on_heat(keystone_session, keystone_auth,
                                        translator, stack_name, file_name,
                                        parsed_params)

                self._write_output(translator, output_file)
        else:
            msg = (_('The path %(template_file)s is not a valid '
                     'file or URL.') % {'template_file': template_file})

            log.error(msg)
            raise ValueError(msg)

    def deploy_on_heat(self, session, auth, translator,
                       stack_name, file_name, parameters):
        endpoint = auth.get_endpoint(session, service_type="orchestration")
        heat_client = heatclient.client.Client('1',
                                               session=session,
                                               auth=auth,
                                               endpoint=endpoint)

        heat_stack_name = stack_name if stack_name else \
            'heat_' + file_name + '_' + str(uuid.uuid4()).split("-")[0]
        msg = _('Deploy the generated template, the stack name is %(name)s.')\
            % {'name': heat_stack_name}
        log.debug(msg)
        tpl = yaml.safe_load(translator.translate())

        # get all the values for get_file from a translated template
        get_files = []
        utils.get_dict_value(tpl, "get_file", get_files)
        files = {}
        if get_files:
            for file in get_files:
                with codecs.open(file, encoding='utf-8', errors='strict') as f:
                    text = f.read()
                    files[file] = text
        tpl['heat_template_version'] = str(tpl['heat_template_version'])
        self._create_stack(heat_client=heat_client,
                           stack_name=heat_stack_name,
                           template=tpl,
                           parameters=parameters,
                           files=files)

    def _create_stack(self, heat_client, stack_name, template, parameters,
                      files):
        if heat_client:
            heat_client.stacks.create(stack_name=stack_name,
                                      template=template,
                                      parameters=parameters,
                                      files=files)

    def _parse_parameters(self, parameter_list):
        parsed_inputs = {}

        # Parameters are semi-colon separated
        inputs = parameter_list.replace('"', '').split(';')
        # Each parameter should be an assignment
        for param in inputs:
            keyvalue = param.split('=')
            # Validate the parameter has both a name and value
            msg = _("'%(param)s' is not a well-formed parameter.") % {
                'param': param}
            if keyvalue.__len__() == 2:
                # Assure parameter name is not zero-length or whitespace
                stripped_name = keyvalue[0].strip()
                if not stripped_name:
                    log.error(msg)
                    raise ValueError(msg)
                # Add the valid parameter to the dictionary
                parsed_inputs[keyvalue[0]] = keyvalue[1]
            else:
                log.error(msg)
                raise ValueError(msg)
        return parsed_inputs

    def _get_translator(self, sourcetype, path, parsed_params, a_file, deploy):
        if sourcetype == "tosca":
            log.debug(_('Loading the tosca template.'))
            tosca = ToscaTemplate(path, parsed_params, a_file)
            csar_dir = None
            if deploy and zipfile.is_zipfile(path):
                # set CSAR directory to the root of TOSCA-Metadata
                csar_decompress = utils.decompress(path)
                csar_dir = os.path.join(csar_decompress,
                                        self.TOSCA_CSAR_META_DIR)
                msg = _("'%(csar)s' is the location of decompressed "
                        "CSAR file.") % {'csar': csar_dir}
                log.info(msg)
            translator = TOSCATranslator(tosca, parsed_params, deploy,
                                         csar_dir=csar_dir)
            log.debug(_('Translating the tosca template.'))
        return translator

    def _write_output(self, translator, output_file=None):
        if output_file:
            path, filename = os.path.split(output_file)
            yaml_files = translator.translate_to_yaml_files_dict(filename)
            for name, content in yaml_files.items():
                with open(os.path.join(path, name), 'w+') as f:
                    f.write(content)
        else:
            print(translator.translate())


def main(args=None):
    if args is None:
        args = sys.argv[1:]
    TranslatorShell().main(args)


if __name__ == '__main__':
    main()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

from toscaparser.common.exception import ExceptionCollector
from toscaparser.utils.gettextutils import _
from translator.common.utils import TranslationUtils
from translator.tests.base import TestCase


class EtsiToscaHotTranslationTest(TestCase):

    def _test_successful_translation(self, tosca_file, hot_files, params=None):
        if not params:
            params = {}
        if not isinstance(hot_files, list):
            hot_files = [hot_files]
        diff = TranslationUtils.compare_tosca_translation_with_hot(tosca_file,
                                                                   hot_files,
                                                                   params)
        self.assertEqual({}, diff, '<difference> : ' +
                         json.dumps(diff, indent=4, separators=(', ', ': ')))

    def _test_failed_translation(self, tosca_file, hot_file, params, msg,
                                 msg_path, error_raise, error_collect):
        if msg_path:
            path = os.path.normpath(os.path.join(
                os.path.dirname(os.path.realpath(__file__)), tosca_file))
            msg = msg % path
        self.assertRaises(
            error_raise,
            TranslationUtils.compare_tosca_translation_with_hot,
            tosca_file, hot_file, params)
        ExceptionCollector.assertExceptionMessage(error_collect, msg)

    def test_hot_translate_etsi_nfv_vnf(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vnf.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_vnf.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vdu(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vdu.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_vdu.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vdu_with_compute_requirements(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vdu_with_compute_requirements.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_vdu_with_compute_requirements.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vdu_with_invalid_compute_requirements(
            self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vdu_with_invalid_compute_requirements.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_vdu_with_invalid_compute_requirements.yaml',
        ]
        expected_msg = _('Unsupported format of '
                         'compute_requirements, '
                         'vdu_name:VDU1, nova_extra_specs:dummy: ][')
        self._test_successful_translation(tosca_file, hot_files, params={})
        self.assertIn(
            expected_msg,
            self.log_fixture.output
        )

    def test_hot_translate_etsi_nfv_vdu_with_logical_node(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vdu_with_logical_node.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_vdu_with_logical_node.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vdu_with_unsupported_storage(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vdu_with_unsupported_storage.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_vdu_with_unsupported_storage.yaml',
        ]
        expected_msg = _('Unsupported virtual_storage, '
                         'vdu_name:VDU1, '
                         'virtual_storage_name:UnsupportedStorage')
        self._test_successful_translation(tosca_file, hot_files, params={})
        self.assertIn(
            expected_msg,
            self.log_fixture.output
        )

    def test_hot_translate_etsi_nfv_cp(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_cp.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_cp.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_cp_with_extended_vnic_type(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_cp_with_extended_vnic_type.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_cp_with_extended_vnic_type.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_check_cp_order(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_check_cp_order.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_check_cp_order.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vl(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vl.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_vl.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vl_with_unsupported_protocol(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vl_with_unsupported_protocol.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_vl_with_unsupported_protocol.yaml',
        ]
        expected_msgs = (
            'Unsupported layer_protocols, virtual_link_name:VL3, '
            'protocol_name:[\'ethernet\']',
            'Unsupported layer_protocols, virtual_link_name:VL4, '
            'protocol_name:[\'mpls\']',
            'Unsupported layer_protocols, virtual_link_name:VL5, '
            'protocol_name:[\'odu2\']',
            'Unsupported layer_protocols, virtual_link_name:VL6, '
            'protocol_name:[\'pseudo-wire\']'
        )
        self._test_successful_translation(tosca_file, hot_files, params={})
        for expected_msg in expected_msgs:
            self.assertIn(
                expected_msg,
                self.log_fixture.output
            )

    def test_hot_translate_etsi_nfv_non_leaf_in_vl(self):
        vl_name = 'VL1'
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_non_leaf_in_vl.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_non_leaf_in_vl.yaml',
        ]
        expected_msgs = (
            'Can not set the required properties '
            'max_kbps on HOT.'
            'virtual_link_name:%s' % vl_name,)
        self._test_successful_translation(tosca_file, hot_files, params={})
        for expected_msg in expected_msgs:
            self.assertIn(
                expected_msg,
                self.log_fixture.output
            )

    def test_hot_translate_etsi_nfv_blockstorage(self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_blockstorage.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'hot_nfv_blockstorage.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vnf_vdu_cp_vl_blockstorage_with_scaling(
            self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vnf_vdu_cp_vl_blockstorage_with_scaling.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'vnf_vdu_cp_vl_blockstorage_with_scaling/'
            'hot_nfv_vnf_vdu_cp_vl_blockstorage_with_scaling.yaml',
            '../tests/data/hot_output/etsi_nfv/'
            'vnf_vdu_cp_vl_blockstorage_with_scaling/'
            'worker_instance.hot.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vdu_cp_vl_with_mixed_scaling(
            self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vdu_cp_vl_with_mixed_scaling.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'vdu_cp_vl_with_mixed_scaling/'
            'hot_nfv_vdu_cp_vl_with_mixed_scaling.yaml',
            '../tests/data/hot_output/etsi_nfv/'
            'vdu_cp_vl_with_mixed_scaling/'
            'worker_instance.hot.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_vdu_cp_with_scaling_multi_aspects(
            self):
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_vdu_cp_with_scaling_multi_aspects.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'vdu_cp_with_scaling_multi_aspects/'
            'hot_nfv_vdu_cp_with_scaling_multi_aspects.yaml',
            '../tests/data/hot_output/etsi_nfv/'
            'vdu_cp_with_scaling_multi_aspects/'
            'worker_instance1.hot.yaml',
            '../tests/data/hot_output/etsi_nfv/'
            'vdu_cp_with_scaling_multi_aspects/'
            'worker_instance2.hot.yaml',
        ]
        self._test_successful_translation(tosca_file, hot_files, params={})

    def test_hot_translate_etsi_nfv_scaling_non_target_vdu_in_initial_delta(
            self):
        aspect_name = 'worker_instance'
        vdu_name = 'VDU1'
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_scaling_non_target_vdu_in_initial_delta.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'scaling_non_target_vdu_in_initial_delta/'
            'hot_nfv_scaling_non_target_vdu_in_initial_delta.yaml',
            '../tests/data/hot_output/etsi_nfv/'
            'scaling_non_target_vdu_in_initial_delta/'
            'worker_instance.hot.yaml',
        ]
        expected_msgs = (
            'Can not set the required properties '
            'min_size and max_size on HOT.'
            'aspect_name:%s' % aspect_name,
        )
        expected_msgs = (
            'No min_size or(and) max_size is found for '
            'aspect_name:%s, VDU:%s' % (
                aspect_name, vdu_name)
        )

        self._test_successful_translation(tosca_file, hot_files, params={})
        for expected_msg in expected_msgs:
            self.assertIn(
                expected_msg,
                self.log_fixture.output
            )

    def test_hot_translate_etsi_nfv_scaling_non_target_vdu_in_aspect_delta(
            self):
        aspect_name = 'worker_instance'
        vdu_name = None
        delta_name = None
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_scaling_non_target_vdu_in_aspect_delta.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'scaling_non_target_vdu_in_aspect_delta/'
            'hot_nfv_scaling_non_target_vdu_in_aspect_delta.yaml',
            '../tests/data/hot_output/etsi_nfv/'
            'scaling_non_target_vdu_in_aspect_delta/'
            'worker_instance.hot.yaml',
        ]
        expected_msgs = (
            'Can not create %s node '
            'because target vdu does not defined.'
            % aspect_name,
            'No ScalingAspectDelta for %s of %s, %s is '
            'found' % (vdu_name, aspect_name, delta_name)
        )
        self._test_successful_translation(tosca_file, hot_files, params={})
        for expected_msg in expected_msgs:
            self.assertIn(
                expected_msg,
                self.log_fixture.output
            )

    def test_hot_translate_etsi_nfv_scaling_non_deltas_in_aspect_delta(self):
        aspect_name = 'worker_instance'
        vdu_name = 'VDU1'
        delta_name = 'delta_2'
        tosca_file = '../tests/data/etsi_nfv/' \
            'tosca_nfv_scaling_non_deltas_in_aspect_delta.yaml'
        hot_files = [
            '../tests/data/hot_output/etsi_nfv/'
            'scaling_non_deltas_in_aspect_delta/'
            'hot_nfv_scaling_non_deltas_in_aspect_delta.yaml',
            '../tests/data/hot_output/etsi_nfv/'
            'scaling_non_deltas_in_aspect_delta/'
            'worker_instance.hot.yaml',
        ]
        expected_msgs = (
            'No ScalingAspectDelta for %s of %s, %s is '
            'found' % (vdu_name, aspect_name, delta_name)
        )
        self._test_successful_translation(tosca_file, hot_files, params={})
        for expected_msg in expected_msgs:
            self.assertIn(
                expected_msg,
                self.log_fixture.output
            )



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from translator.hot.syntax.hot_resource import HotResource

# Name used to dynamically load appropriate map class.
TARGET_CLASS_NAME = 'ToscaSoftwareComponent'


class ToscaSoftwareComponent(HotResource):
    '''Translate TOSCA node type tosca.nodes.SoftwareComponent.'''

    toscatype = 'tosca.nodes.SoftwareComponent'

    def __init__(self, nodetemplate, csar_dir=None):
        super(ToscaSoftwareComponent, self).__init__(nodetemplate,
                                                     csar_dir=csar_dir)
        pass

    def handle_properties(self):
        pass



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from collections import OrderedDict
import logging
from toscaparser.utils.gettextutils import _

KEYS = (TYPE, DESCRIPTION, DEFAULT, CONSTRAINTS, HIDDEN, LABEL) = \
       ('type', 'description', 'default', 'constraints', 'hidden', 'label')

log = logging.getLogger('heat-translator')


class HotParameter(object):
    '''Attributes for HOT parameter section.'''

    def __init__(self, name, type, label=None, description=None, default=None,
                 hidden=None, constraints=None):
        self.name = name
        self.type = type
        self.label = label
        self.description = description
        self.default = default
        self.hidden = hidden
        self.constraints = constraints
        log.info(_('Initialized the input parameters.'))

    def get_dict_output(self):
        param_sections = OrderedDict()
        param_sections[TYPE] = self.type
        if self.label:
            param_sections[LABEL] = self.label
        if self.description:
            param_sections[DESCRIPTION] = self.description
        if self.default:
            param_sections[DEFAULT] = self.default
        if self.hidden:
            param_sections[HIDDEN] = self.hidden
        if self.constraints:
            param_sections[CONSTRAINTS] = self.constraints

        return {self.name: param_sections}
